'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = diffTree;

var _ChildOperationTypes = require('./ChildOperationTypes');

function indexOf(nodes, node, isSame, nodeIndex) {
  var len = nodes.length;
  for (var i = 0; i < len; i++) {
    if (isSame(node, nodes[i], nodeIndex, i)) {
      return i;
    }
  }
  return -1;
}

function nativeShould(a, b) {
  return a === b;
}

function sortByIndex(a, b) {
  if (a.fromIndex === b.fromIndex) {
    return 0;
  }
  return a.fromIndex > b.fromIndex ? -1 : 1;
}

// diff by level
function diff(fromNodes, afterNodes) {
  var options = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];
  var internal = arguments.length <= 3 || arguments[3] === undefined ? {} : arguments[3];
  var _options$shouldUpdate = options.shouldUpdate;
  var shouldUpdate = _options$shouldUpdate === undefined ? nativeShould : _options$shouldUpdate;
  var _options$childrenKey = options.childrenKey;
  var childrenKey = _options$childrenKey === undefined ? 'children' : _options$childrenKey;
  var _internal$fromPath = internal.fromPath;
  var fromPath = _internal$fromPath === undefined ? [] : _internal$fromPath;
  var parentNode = internal.parentNode;

  var insertQueue = [];
  var updateQueue = [];
  var removeQueue = [];
  var lastIndex = 0;
  var tmp = void 0;
  afterNodes.forEach(function (afterNode, toIndex) {
    var fromIndex = indexOf(fromNodes, afterNode, shouldUpdate, toIndex);
    if (fromIndex !== -1) {
      var fromNode = fromNodes[fromIndex];
      updateQueue.push({
        type: _ChildOperationTypes.UPDATE,
        fromNode: fromNode,
        afterNode: afterNode,
        parentNode: parentNode,
        fromIndex: fromIndex,
        fromPath: fromPath.concat(fromIndex)
      });
      if (fromIndex < lastIndex) {
        tmp = {
          type: _ChildOperationTypes.MOVE,
          fromNode: fromNode,
          afterNode: afterNode,
          parentNode: parentNode,
          fromIndex: fromIndex,
          toIndex: toIndex,
          fromPath: fromPath.concat(fromIndex),
          toPath: fromPath.concat(toIndex)
        };
        insertQueue.push(tmp);
        removeQueue.push(tmp);
      }
      lastIndex = Math.max(fromIndex, lastIndex);
    } else {
      insertQueue.push({
        type: _ChildOperationTypes.NEW,
        afterNode: afterNode,
        parentNode: parentNode,
        toIndex: toIndex,
        toPath: fromPath.concat(toIndex)
      });
    }
  });

  fromNodes.forEach(function (fromNode, fromIndex) {
    var toIndex = indexOf(afterNodes, fromNode, shouldUpdate, fromIndex);
    if (toIndex === -1) {
      removeQueue.push({
        type: _ChildOperationTypes.REMOVE,
        fromNode: fromNode,
        parentNode: parentNode,
        fromIndex: fromIndex,
        fromPath: fromPath.concat(fromIndex)
      });
    }
  });

  removeQueue.sort(sortByIndex);

  if (childrenKey) {
    updateQueue.concat().forEach(function (o) {
      var currentChildren = o.fromNode[childrenKey] || [];
      var nextChildren = o.afterNode[childrenKey] || [];
      // bottom up
      var ret = diff(currentChildren, nextChildren, options, {
        fromPath: o.fromPath,
        parentNode: o.fromNode
      });
      insertQueue = ret.insertQueue.concat(insertQueue);
      updateQueue = ret.updateQueue.concat(updateQueue);
      removeQueue = ret.removeQueue.concat(removeQueue);
    });
  }

  return {
    insertQueue: insertQueue,
    updateQueue: updateQueue,
    removeQueue: removeQueue
  };
}

function diffTree(fromNodes, afterNodes, options) {
  return diff(fromNodes, afterNodes, options);
}
module.exports = exports['default'];