'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _patch = require('../patch');

var _patch2 = _interopRequireDefault(_patch);

var _createDOMNode = require('./createDOMNode');

var _createDOMNode2 = _interopRequireDefault(_createDOMNode);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function findNodeByPath(root, fromPath) {
  var parent = root;
  var node = fromPath.reduce(function (n, p) {
    parent = n;
    return n.childNodes[p];
  }, root) || null;
  return {
    node: node,
    parent: parent
  };
}

function processNew(root, q) {
  var afterNode = q.afterNode;
  var toPath = q.toPath;

  var newNode = (0, _createDOMNode2["default"])(afterNode);

  var _findNodeByPath = findNodeByPath(root, toPath);

  var parent = _findNodeByPath.parent;
  var node = _findNodeByPath.node;

  parent.insertBefore(newNode, node);
}

function processMove(_, q, _ref) {
  var parent = _ref.parent;
  var node = _ref.node;
  var toPath = q.toPath;

  parent.insertBefore(node, parent.childNodes[toPath[toPath.length - 1]] || null);
}

function processRemove(root, q) {
  var fromPath = q.fromPath;

  var _findNodeByPath2 = findNodeByPath(root, fromPath);

  var parent = _findNodeByPath2.parent;
  var node = _findNodeByPath2.node;

  parent.removeChild(node);
  return { parent: parent, node: node };
}

function processUpdate(root, q) {
  var fromPath = q.fromPath;
  var fromNode = q.fromNode;
  var afterNode = q.afterNode;

  var _findNodeByPath3 = findNodeByPath(root, fromPath);

  var node = _findNodeByPath3.node;

  if (typeof afterNode === 'string') {
    if (afterNode !== fromNode) {
      node.nodeValue = afterNode;
    }
    return;
  }
  var currentProps = fromNode.props;
  var nextProps = afterNode.props;
  for (var nextName in nextProps) {
    if (nextProps.hasOwnProperty(nextName)) {
      var nextValue = nextProps[nextName];
      var currentValue = currentProps[nextName];
      if (nextValue !== currentValue) {
        node.setAttribute(nextName, nextValue);
      }
    }
  }
  for (var currentName in currentProps) {
    if (currentProps.hasOwnProperty(currentName) && !nextProps.hasOwnProperty(currentName)) {
      node.removeAttribute(currentName);
    }
  }
}

function patchDOM(queue, root) {
  (0, _patch2["default"])(queue, {
    processNew: processNew.bind(null, root),
    processMove: processMove.bind(null, root),
    processUpdate: processUpdate.bind(null, root),
    processRemove: processRemove.bind(null, root)
  });
}

exports["default"] = patchDOM;
module.exports = exports['default'];